"""
The :mod:`skcycling.utils` module include utility functions.
"""

# Authors: Guillaume Lemaitre <g.lemaitre58@gmail.com>
#          Cedric Lemaitre
# License: BSD 3 clause

from .validation import validate_filenames


__all__ = ['validate_filenames']
