"""
The :mod:`skcycling.io` module includes utility to load data.
"""

# Authors: Guillaume Lemaitre <g.lemaitre58@gmail.com>
#          Cedric Lemaitre
# License: BSD 3 clause

from .base import bikeread

__all__ = ['bikeread']
